@@ -12,7 +12,6 @@ import warnings
 from collections import deque
 from contextlib import contextmanager
 
-from functools import lru_cache
 from functools import partial
 from functools import reduce
 
@@ -544,7 +543,6 @@ class Context:
                 return self.env.undefined(name)
             return default
 
-    @lru_cache(maxsize=128)
     def filter(self, name: str) -> Callable[..., object]:
         """Return the filter function with given name."""
         try:
@@ -666,11 +664,8 @@ class Context:
             self.env.loop_iteration_limit
             and reduce(
                 mul,
-                itertools.chain(
-                    (loop.length for loop in self.loops),
-                    [length, self.loop_iteration_carry],
-                ),
-                1,
+                (loop.length for loop in self.loops),
+                length * self.loop_iteration_carry,
             )
             > self.env.loop_iteration_limit
         ):
@@ -690,7 +685,11 @@ class Context:
             )
 
         loop_iteration_carry = (
-            reduce(mul, (loop.length for loop in self.loops), 1)
+            reduce(
+                mul,
+                (loop.length for loop in self.loops),
+                self.loop_iteration_carry,
+            )
             if carry_loop_iterations
             else 1
         )
