@@ -1,3 +1,6 @@
+"""Text processing utilities."""
+
+
 def truncate_chars(val: str, num: int, end: str = "...") -> str:
     """Truncates a string if it is longer than the specified number of characters.
     Truncated strings will end with `end`, an ellipsis by default."""
@@ -12,7 +15,7 @@ def truncate_chars(val: str, num: int, end: str = "...") -> str:
 
 def truncate_words(val: str, num: int, end: str = "...") -> str:
     """Truncates a string down to the number of words passed as the first parameter."""
-    # XXX: Replaces consecutive whitespace with a single newline.
+    # Replaces consecutive whitespace with a single newline.
     words = val.split()
 
     if len(words) < num:
